import django_tables2 as tables
from CustomAuth.models import User


class UserTable(tables.Table):
    class Meta:
        model = User
        template_name = "django_tables2/bootstrap.html"
        fields = ('email', 'full_name', 'is_verify', 'is_staff', 'is_superuser', 'jalali_last_login',)



from django.shortcuts import render


def page_not_found(request, exception=None):
    context = {
        'title': 'Page not found',
        'text': 'Sorry page not found',
        'code': 404
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)


def server_error(request, exception=None):
    context = {
        'title': 'Server error',
        'text': 'Sorry the server is unavailable',
        'code': 500
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)


def permission_denied(request, exception=None):
    context = {
        'title': 'Forbidden',
        'text': 'Permission Denied',
        'code': 403
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)


def unauthorized(request, exception=None):
    context = {
        'title': 'Unauthorized',
        'text': 'Unauthorized',
        'code': 401
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)


def bad_request(request, exception=None):
    context = {
        'title': 'Bad request',
        'text': 'Bad request',
        'code': 400
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)


def maintenance(request, exception=None):
    context = {
        'title': 'Server on Maintenance',
        'text': "Sorry we're down for maintenance",
        'code': 503
    }
    return render(request, 'CustomAuth/pages/status.html', context=context)



from django.contrib.auth.base_user import BaseUserManager
from django.utils import timezone
from datetime import datetime
import jwt
from django.conf import settings


class UserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError('The email must be set')

        email = self.normalize_email(email=email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        databases = getattr(settings, 'AUTH_DATABASES', [])
        if databases:
            for db in databases:
                user.save(using=db)
        else:
            user.save(using=self.db)

        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Creates and saves a superuser with the given email and password.
        """

        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)
        extra_fields.setdefault('is_verify', True)
        extra_fields.setdefault('date_verify', datetime.now())

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self.create_user(email, password, **extra_fields)

    def get_by_token(self, token: str, algorithm='HS256'):
        encoded_token = token.encode('utf-8')
        try:
            decoded: dict = jwt.decode(encoded_token, settings.SECRET_KEY, algorithm)
            user_id = decoded.get('id', None)
            if user_id:
                user = self.get(pk=user_id)
                return user
            user_email = decoded.get('email', None)
            if user_email:
                user = self.get(email=user_email)
                return user
            user_username = decoded.get('username', None)
            if user_username:
                user = self.get(username=user_username)
                return user
        except jwt.ExpiredSignatureError:
            return None  # Signature expired. Please log in again.
        except jwt.InvalidTokenError:
            return None  # Invalid token. Please log in again.
        except self.model.DoesNotExist:
            return None  # User does not exist


class SuperuserManager(BaseUserManager):
    def get_queryset(self):
        return super().get_queryset().filter(is_superuser=True)

    def create(self, email, password, **extra_fields):
        return self.create_user(email, password, **extra_fields)

    def create_user(self, email, password, **extra_fields):
        """
        Creates and saves a superuser with the given email and password.
        """

        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)
        extra_fields.setdefault('is_verify', True)
        extra_fields.setdefault('date_verify', timezone.now())

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        if not email:
            raise ValueError('The email must be set')

        email = self.normalize_email(email=email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        databases = getattr(settings, 'AUTH_DATABASES', [])
        if databases:
            for db in databases:
                user.save(using=db)
        else:
            user.save(using=self.db)

        return user

    def get_by_token(self, token: str, algorithm='HS256'):
        encoded_token = token.encode('utf-8')
        try:
            decoded: dict = jwt.decode(encoded_token, settings.SECRET_KEY, algorithm)
            user_id = decoded.get('id', None)
            if user_id:
                user = self.get(pk=user_id)
                return user
            user_email = decoded.get('email', None)
            if user_email:
                user = self.get(email=user_email)
                return user
            user_username = decoded.get('username', None)
            if user_username:
                user = self.get(username=user_username)
                return user
        except jwt.ExpiredSignatureError:
            return None  # Signature expired. Please log in again.
        except jwt.InvalidTokenError:
            return None  # Invalid token. Please log in again.
        except self.model.DoesNotExist:
            return None  # User does not exist


class StaffManager(BaseUserManager):
    def get_queryset(self):
        return super().get_queryset().filter(is_staff=True)

    def create(self, email, password, **extra_fields):
        return self.create_user(email, password, **extra_fields)

    def create_user(self, email, password, **extra_fields):
        """
        Creates and saves a superuser with the given email and password.
        """

        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_active', True)
        extra_fields.setdefault('is_verify', True)
        extra_fields.setdefault('date_verify', timezone.now())

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')

        if not email:
            raise ValueError('The email must be set')

        email = self.normalize_email(email=email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        databases = getattr(settings, 'AUTH_DATABASES', [])
        if databases:
            for db in databases:
                user.save(using=db)
        else:
            user.save(using=self.db)

        return user

    def get_by_token(self, token: str, algorithm='HS256'):
        encoded_token = token.encode('utf-8')
        try:
            decoded: dict = jwt.decode(encoded_token, settings.SECRET_KEY, algorithm)
            user_id = decoded.get('id', None)
            if user_id:
                user = self.get(pk=user_id)
                return user
            user_email = decoded.get('email', None)
            if user_email:
                user = self.get(email=user_email)
                return user
            user_username = decoded.get('username', None)
            if user_username:
                user = self.get(username=user_username)
                return user
        except jwt.ExpiredSignatureError:
            return None  # Signature expired. Please log in again.
        except jwt.InvalidTokenError:
            return None  # Invalid token. Please log in again.
        except self.model.DoesNotExist:
            return None  # User does not exist


from django.http import HttpRequest, HttpResponseRedirect
from CustomAuth.forms import UserLoginForm
from django.contrib.auth import login as user_login
from django.conf import settings
from django.shortcuts import render


def login(request: HttpRequest):
    if not request.user.is_anonymous:
        url = getattr(settings, 'USER_PROFILE_URL', '/profile/')
        return HttpResponseRedirect(url)
    if request.method == 'POST':
        form = UserLoginForm(data=request.POST)
        if form.is_valid():
            form.clean()
            user_login(request, form.get_user(), backend='django.contrib.auth.backends.ModelBackend')
            next_page = request.session.get('next', None)
            if next_page:
                url = next_page
            else:
                url = getattr(settings, 'USER_PROFILE_URL', '/profile/')
            return HttpResponseRedirect(url)
        else:
            context = {
                'errors': form.errors,
                'form': UserLoginForm
            }
    else:
        context = {
            'form': UserLoginForm,
        }
    if request.method == 'GET':
        next_page = request.GET.get('next', None)
        if next_page:
            request.session['next'] = next_page
    return render(request, 'CustomAuth/pages/login.html', context=context)



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.utils.translation import gettext_lazy as _
from .forms.UserForms import UserChangeForm, UserCreationForm
from .models import User
from django.contrib.auth.models import Group, Permission


class UserAdmin(BaseUserAdmin):
    form = UserChangeForm
    add_form = UserCreationForm

    fieldsets = (
        (
            None, {
                'fields': (
                    'username',
                    'password'
                )
            }
        ),
        (
            _('Personal info'), {
                'fields': (
                    'first_name',
                    'last_name',
                    'email'
                )
            }
        ),
        (
            _('finance'), {
                'fields': (
                    'wallet',
                )
            }
        ),
        (
            _('Permissions'), {
                'fields': (
                    'is_active',
                    'is_verify',
                    'is_staff',
                    'is_superuser',
                    'groups',
                    'user_permissions'
                ),
            }
        ),
        (
            _('Important dates'), {
                'fields': (
                    'date_verify',
                    'last_login',
                )
            }
        ),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('email', 'password1', 'password2'),
        }),
    )

    list_display = ('email', 'username', 'first_name', 'last_name', 'is_verify')

    list_filter = ('is_staff', 'is_superuser', 'is_active', 'groups', 'is_verify')

    search_fields = ('username', 'first_name', 'last_name', 'email', 'groups')

    ordering = ('username',)

    filter_horizontal = ('groups', 'user_permissions',)


admin.site.register(User, UserAdmin)
admin.site.register(Permission)




import re


class ProfileMixin():

    def __getattr__(self, name):
        pattern = re.compile(r'^is_(.+)$')
        match = re.match(pattern, name)
        if match:
            return self.is_profile(match.group(1))

    def is_profile(self, name):
        return bool(getattr(self, name, None))



from django.http import HttpRequest, JsonResponse
from CustomAuth.models import User
from django.contrib.auth.decorators import login_required


@login_required
def new_jwt_token(request: HttpRequest):
    user: User = request.user
    token = user.token
    response = {
        'jwt-authentication': token
    }
    return JsonResponse(response, status=200)



# Generated by Django 3.0 on 2019-12-09 15:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('CustomAuth', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='date_joined',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='join date'),
        ),
    ]



# Generated by Django 3.0 on 2019-12-09 15:08

import django.contrib.auth.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('date_register', models.DateTimeField(blank=True, null=True, verbose_name='register date')),
                ('date_joined', models.DateTimeField(blank=True, null=True, verbose_name='join date')),
                ('is_active', models.BooleanField(default=True, verbose_name='active status')),
                ('is_register', models.BooleanField(default=False, verbose_name='register status')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates whether this user should be treated as active.Unselect this instead of deleting accounts.')),
                ('first_name', models.CharField(blank=True, max_length=100, null=True, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=200, null=True, verbose_name='first name')),
                ('username', models.CharField(blank=True, error_messages={'unique': 'A user with that username already exists.'}, help_text='150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, null=True, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('email', models.EmailField(error_messages={'unique': 'A user with that email already exists.'}, help_text='Required, 250 characters or fewer.', max_length=250, unique=True, verbose_name='email address')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='users', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='users', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
                'swappable': 'AUTH_USER_MODEL',
            },
        ),
    ]



import time


def timeit(function):
    def timed(*args, **kwargs):
        ts = time.time()
        result = function(*args, **kwargs)
        te = time.time()
        print('%r (%r, %r) %2.2f sec' % (function.__name__, args, kwargs, te - ts))
        return result

    return timed



from django.http import HttpRequest, JsonResponse
from CustomAuth.models import User


def jwt(function):
    def wrap(request: HttpRequest, *args, **kwargs):
        user: User = request.user
        is_jwt_authentication = request.headers.__contains__('jwt-authentication')
        if not user.is_anonymous and is_jwt_authentication:
            return function(request, *args, *kwargs)
        else:
            context = {
                'error': 'jwt authentication is required'
            }
            return JsonResponse(context, status=403)

    return wrap



# Generated by Django 3.0 on 2019-12-18 10:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('CustomAuth', '0004_auto_20191217_0409'),
    ]

    operations = [
        migrations.RenameField(
            model_name='user',
            old_name='permissions',
            new_name='user_permissions',
        ),
    ]



from django.http import HttpRequest, HttpResponseRedirect
from CustomAuth.forms import UserCreationForm
from django.contrib.auth import login
from django.conf import settings
from django.shortcuts import render


def signup(request: HttpRequest):
    if request.method == 'POST':
        form = UserCreationForm(data=request.POST)
        if form.is_valid() and form.clean_password2():
            user = form.save()
            user.save()
            login(request, user, backend='django.contrib.auth.backends.ModelBackend')
            user.verification_email(request)
            return HttpResponseRedirect(getattr(settings, 'SIGNUP_SUCCESSFULLY_URL', '/profile/'))
        else:
            context = {
                'errors': form.errors,
                'form': UserCreationForm,
            }
    else:
        context = {
            'form': UserCreationForm
        }
    return render(request, 'CustomAuth/pages/signup.html', context)



from .abstract_user import AbstractUser


class User(AbstractUser):
    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta(AbstractUser.Meta):
        swappable = 'AUTH_USER_MODEL'



from .anonymous import anonymous_required
from .group import group_required
from .superuser import superuser_only
from .ajax import ajax_required
from .timer import timeit
from .jwt import jwt as jwt_required

# TODO : Convert to class based decorator



from .date_mixin import DateMixin
from .permission_mixin import PermissionMixin
from .abstract_user import AbstractUser
from .user import User
from .profile import Profile

from django.utils import timezone


def update_last_login(sender, auth_user, **kwargs):
    """
    A signal receiver which updates the last_login date for
    the user logging in.
    """
    auth_user.last_login = timezone.now()
    auth_user.save(update_fields=['last_login'])



from django.contrib.auth.decorators import user_passes_test
from django.conf import settings


def group_required(*group_names, redirect_url=None):
    """
    Requires user membership in at least one of the groups passed in.
    The way to use this decorator is:
        @group_required(‘admins’, ‘seller’)
        def my_view(request, pk)
        ...
    """
    """
    To support modern django rest framework APIViews (class-based views).
    @method_decorator(group_required('groupa', 'groupb'))
    """

    if not redirect_url:
        redirect_url = settings.LOGIN_REDIRECT_URL

    def in_groups(user):
        if user.is_authenticated:
            if bool(user.groups.filter(name__in=group_names)) | user.is_superuser:
                return True
        return False

    actual_decorator = user_passes_test(
        in_groups,
        login_url=redirect_url
    )

    return actual_decorator



from django.db import models
from django.utils.translation import gettext_lazy as _


class FinanceMixin(models.Model):
    wallet = models.PositiveIntegerField(
        _('Credit of user'),
        default=0
    )

    class Meta:
        abstract = True



from django.db import models
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.base_user import AbstractBaseUser
from .permission_mixin import PermissionMixin
from .date_mixin import DateMixin
from django.core.mail import send_mail
from django.utils import timezone
from CustomAuth.managers import UserManager, SuperuserManager, StaffManager
from .finance_mixin import FinanceMixin
from .profile import Profile
from django.contrib.sites.shortcuts import get_current_site
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.http import urlsafe_base64_encode
from django.utils.encoding import force_bytes
from CustomAuth.tokens import account_verify_email_token, magic_token
from smtplib import SMTPException
import jwt
from jdatetime import datetime as jalali


class AbstractUser(AbstractBaseUser, PermissionMixin, DateMixin, FinanceMixin):
    username_validator = UnicodeUsernameValidator()

    first_name = models.CharField(
        _('first name'),
        max_length=100,
        blank=True,
        null=True,
    )
    last_name = models.CharField(
        _('last name'),
        max_length=200,
        blank=True,
        null=True,
    )

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        blank=True,
        null=True,
        help_text=_('150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        }
    )

    email = models.EmailField(
        _('email address'),
        max_length=250,
        unique=True,
        help_text=_('Required, 250 characters or fewer.'),
        error_messages={
            'unique': _("A user with that email already exists."),
        }
    )

    objects = UserManager()
    superusers = SuperuserManager()
    staff = StaffManager()

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    @property
    def full_name(self):
        return self.get_full_name()

    @property
    def jalali_last_login(self):
        return jalali.fromgregorian(datetime=self.last_login).strftime("%d %m %Y")

    def __str__(self):
        if self.username is None:
            return "%s" % self.email
        return "@%s : %s" % (self.username, self.email)

    def register_user(self):
        self.data_register = timezone.now()
        self.is_verify = True
        self.save(update_fields=['date_register', 'is_register'])

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        if (not self.first_name) or (not self.last_name):
            full_name = '---'
        else:
            full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def profile(self):
        if not hasattr(self, '_profile') or not self._profile:
            self._profile = None
            for field in self._meta.get_fields():
                field_name = field.name
                value = getattr(self, field_name, None)
                if value and isinstance(value, Profile):
                    self._profile = value
        return self._profile

    def verification_email(self, request):
        current_site = get_current_site(request)
        mail_subject = getattr(settings, 'REGISTRATION_MAIL_SUBJECT', 'Activate your blog account.')
        context = {
            'user': self,
            'domain': current_site.domain,
            'uid': urlsafe_base64_encode(force_bytes(self.pk)),
            'token': account_verify_email_token.make_token(self)
        }
        message = render_to_string('CustomAuth/pages/email_verification.html', context=context)
        try:
            self.email_user(mail_subject, message, from_email=settings.EMAIL_FROM)
        except (SMTPException, Exception):
            print(message)

    def get_magic_link(self, request):
        current_site = get_current_site(request)
        uidb64 = urlsafe_base64_encode((force_bytes(self.pk)))
        token = magic_token.make_token(self)
        magic_link = 'http://{}/{}/{}'.format(current_site.domain, uidb64, token)
        print(magic_link)
        return magic_link

    @property
    def token(self):
        return self.generate_jwt_token()
        pass

    def generate_jwt_token(self):
        """
        Generates a JSON Web Token that stores this user's ID and has an expiry
        date set to 30 days into the future.
        """
        dt = timezone.datetime.now() + timezone.timedelta(days=30)

        token = jwt.encode(
            payload={
                'id': self.pk,
                'exp': dt
            },
            key=settings.SECRET_KEY, algorithm='HS256')

        return token.decode('utf-8')



from django.http import HttpRequest
from django.shortcuts import render
from django_tables2 import SingleTableView
from django_tables2.paginators import LazyPaginator
from django_tables2.config import RequestConfig
from CustomAuth.tables import UserTable
from CustomAuth.models import User
from CustomAuth.decorators import superuser_only


@superuser_only
def user_list(request: HttpRequest):
    table = UserTable(User.objects.all())
    RequestConfig(request=request, paginate={"paginator_class": LazyPaginator}).configure(table)
    context = {
        'table': table
    }
    return render(request, 'CustomAuth/pages/tables.html', context=context)


@superuser_only
def superuser_list(request: HttpRequest):
    table = UserTable(User.superusers.all())
    RequestConfig(request=request, paginate={"paginator_class": LazyPaginator}).configure(table)
    context = {
        'table': table
    }
    return render(request, 'CustomAuth/pages/tables.html', context=context)


@superuser_only
def staff_list(request: HttpRequest):
    table = UserTable(User.staff.all())
    RequestConfig(request=request, paginate={"paginator_class": LazyPaginator}).configure(table)
    context = {
        'table': table
    }
    return render(request, 'CustomAuth/pages/tables.html', context=context)



from django.db import models
from django.utils.translation import gettext_lazy as _

from django.contrib.auth.models import Group, Permission

from django.contrib import auth
from django.core.exceptions import PermissionDenied


def _user_get_permissions(user, obj, from_name):
    permissions = set()
    name = 'get_%s_permissions' % from_name
    for backend in auth.get_backends():
        if hasattr(backend, name):
            permissions.update(getattr(backend, name)(user, obj))
    return permissions


def _user_has_perm(user, perm, obj):
    """
    A backend can raise `PermissionDenied` to short-circuit permission checking.
    """
    for backend in auth.get_backends():
        if not hasattr(backend, 'has_perm'):
            continue
        try:
            if backend.has_perm(user, perm, obj):
                return True
        except PermissionDenied:
            return False
    return False


def _user_has_module_perms(user, app_label):
    """
    A backend can raise `PermissionDenied` to short-circuit permission checking.
    """
    for backend in auth.get_backends():
        if not hasattr(backend, 'has_module_perms'):
            continue
        try:
            if backend.has_module_perms(user, app_label):
                return True
        except PermissionDenied:
            return False
    return False


class PermissionMixin(models.Model):
    is_active = models.BooleanField(
        _('active status'),
        default=True
    )
    is_verify = models.BooleanField(
        _('register status'),
        default=False
    )
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text='Designates whether the user can log into this admin site.'
    )
    is_superuser = models.BooleanField(
        default=False,
        help_text='Designates whether this user should be treated as active.' +
                  'Unselect this instead of deleting accounts.'
    )

    groups = models.ManyToManyField(
        Group,
        verbose_name=_('groups'),
        blank=True,
        help_text=_(
            'The groups this user belongs to. A user will get all permissions '
            'granted to each of their groups.'
        ),
        related_name='users',
        related_query_name='user'
    )

    user_permissions = models.ManyToManyField(
        Permission,
        verbose_name=_('user permissions'),
        blank=True,
        help_text=_('Specific permissions for this user.'),
        related_name="users",
        related_query_name="user",
    )

    class Meta:
        abstract = True

    def get_user_permissions(self, obj=None):
        """
        Return a list of permission strings that this user has directly.
        Query all available auth backends. If an object is passed in,
        return only permissions matching this object.
        """
        return _user_get_permissions(self, obj, 'user')

    def get_group_permissions(self, obj=None):
        """
        Return a list of permission strings that this user has through their
        groups. Query all available auth backends. If an object is passed in,
        return only permissions matching this object.
        """
        return _user_get_permissions(self, obj, 'group')

    def get_all_permissions(self, obj=None):
        return _user_get_permissions(self, obj, 'all')

    def has_perm(self, perm, obj=None):
        """
        Return True if the user has the specified permission. Query all
        available auth backends, but return immediately if any backend returns
        True. Thus, a user who has permission from a single auth backend is
        assumed to have permission in general. If an object is provided, check
        permissions for that object.
        """
        # Active superusers have all permissions.
        if self.is_active and self.is_superuser:
            return True

        # Otherwise we need to check the backends.
        return _user_has_perm(self, perm, obj)

    def has_perms(self, perm_list, obj=None):
        """
        Return True if the user has each of the specified permissions. If
        object is passed, check if the user has all required perms for it.
        """
        return all(self.has_perm(perm, obj) for perm in perm_list)

    def has_module_perms(self, app_label):
        """
        Return True if the user has any permissions in the given app label.
        Use similar logic as has_perm(), above.
        """
        # Active superusers have all permissions.
        if self.is_active and self.is_superuser:
            return True

        return _user_has_module_perms(self, app_label)



from django.urls import path
from CustomAuth.views import user_list, superuser_list, staff_list
from django.views.generic.base import RedirectView
urlpatterns = [
    path("user/list/", RedirectView.as_view(url="all")),
    path("user/list/all", user_list),
    path("user/list/superuser", superuser_list),
    path("user/list/staff", staff_list)
]



from .handler import handler400, handler401, handler403, handler404, handler500
from django.conf.urls import url
from CustomAuth.views import login, verify_email, signup, logout, resend_verification_code

from .password import urlpatterns as password_patterns
from .jwt import urlpatterns as jwt_patterns
from .social import urlpatterns as social_patterns
from .tables import urlpatterns as table_patterns

urlpatterns = [
    url('login/', login, name='login'),
    url('signup/', signup, name='signup'),
    url('logout/', logout, name='logout'),
    url('verify/resend/', resend_verification_code, name='resend verify'),
    url(r'^verify/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$', verify_email,
        name='verify'),
]

urlpatterns += password_patterns
urlpatterns += jwt_patterns
urlpatterns += social_patterns
urlpatterns += table_patterns



from CustomAuth.models import User
from CustomAuth.tokens import account_verify_email_token
from django.contrib.auth import login
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.encoding import force_text
from django.utils.http import urlsafe_base64_decode
from django.conf import settings


def verify_email(request, uidb64, token):
    try:
        uid = force_text(urlsafe_base64_decode(uidb64))
        user = User.objects.get(pk=uid)
    except(TypeError, ValueError, OverflowError, User.DoesNotExist):
        user = None
    if user is not None and account_verify_email_token.check_token(user, token):
        user.is_verify = True
        user.save()
        login(request, user, backend='django.contrib.auth.backends.ModelBackend')
        return HttpResponseRedirect(getattr(settings, 'VERIFY_SUCCESSFULLY', '/profile/'))
    else:
        return HttpResponse(getattr(settings, 'VERIFY_FAILED', 'Verification link is invalid!'))



from .mixin import MiddlewareMixin
from django.http import HttpRequest, HttpResponse
from CustomAuth.models import User


class JwtMiddleware(MiddlewareMixin):
    def __init__(self, get_response):
        super(JwtMiddleware, self).__init__()
        self.get_response = get_response

    def process_request(self, request: HttpRequest):
        if not request.user or request.user.is_anonymous:
            jwt_token = request.headers.get('jwt-authentication', None)
            print(jwt_token)
            if jwt_token:
                user = User.objects.get_by_token(jwt_token)
                if user:
                    request.user = user

    def process_response(self, request: HttpRequest, response: HttpResponse):
        return response



from django.db import models
from django.conf import settings
from django.utils.translation import gettext_lazy as _


class Profile(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        verbose_name=_('profile'),
    )

    class Meta:
        abstract = True



from CustomAuth.backends import MagicLinkBackend
from CustomAuth.middleware import MiddlewareMixin
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
from django.contrib.auth import login
from django.conf import settings


class MagicMiddleware(MiddlewareMixin):
    def __init__(self, get_response):
        super(MagicMiddleware, self).__init__()
        self.get_response = get_response
        self.magic_link_backend = MagicLinkBackend()

    def process_request(self, request: HttpRequest):
        pass

    def process_response(self, request: HttpRequest, response: HttpResponse):
        return response

    def process_view(self, request: HttpRequest, view_func, view_args: list, view_kwargs: dict):
        uidb64 = view_kwargs.get('uidb64', None)
        token = view_kwargs.get('token', None)
        if uidb64 and token:
            backend = MagicLinkBackend()
            user = backend.authenticate(request, uidb64, token)
            if user:
                login(request, user)
                profile_url = getattr(settings, 'USER_PROFILE_URL', '/profile/')
                return HttpResponseRedirect(profile_url)
        return None



from django.core.exceptions import PermissionDenied


def superuser_only(function):
    """Limit view to superusers only."""

    def wrap(request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
        return function(request, *args, **kwargs)

    return wrap



# Generated by Django 3.0 on 2019-12-17 00:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('CustomAuth', '0003_user_wallet'),
    ]

    operations = [
        migrations.RenameField(
            model_name='user',
            old_name='is_register',
            new_name='is_verify',
        ),
        migrations.AlterField(
            model_name='user',
            name='last_name',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='last name'),
        ),
    ]



from django.conf.urls import url
from authlib import views
from django.conf import settings

urlpatterns = []

if getattr(settings, 'GOOGLE_CLIENT_ID', '') and getattr(settings, 'GOOGLE_CLIENT_SECRET', ''):
    from authlib.google import GoogleOAuth2Client
    urlpatterns += url(
        r"^oauth/google/$",
        views.oauth2,
        {
            "client_class": GoogleOAuth2Client,
        },
        name="accounts_oauth_google",
    ),

if getattr(settings, 'FACEBOOK_CLIENT_ID', '') and getattr(settings, 'FACEBOOK_CLIENT_SECRET', ''):
    from authlib.facebook import FacebookOAuth2Client
    urlpatterns += url(
        r"^oauth/facebook/$",
        views.oauth2,
        {
            "client_class": FacebookOAuth2Client,
        },
        name="accounts_oauth_facebook",
    ),

if getattr(settings, 'TWITTER_CLIENT_ID', '') and getattr(settings, 'TWITTER_CLIENT_SECRET', ''):
    from authlib.twitter import TwitterOAuthClient
    urlpatterns += url(
        r"^oauth/twitter/$",
        views.oauth2,
        {
            "client_class": TwitterOAuthClient,
        },
        name="accounts_oauth_twitter",
    )



from django.contrib.auth.backends import BaseBackend
from CustomAuth.models import User
from django.utils.http import urlsafe_base64_decode
from django.utils.encoding import force_text
from CustomAuth.tokens import magic_token
from django.conf import settings


class MagicLinkBackend(BaseBackend):
    def get_user(self, uidb64):
        try:
            uid = force_text(urlsafe_base64_decode(uidb64))
            user = User.objects.get(pk=uid)
        except(TypeError, ValueError, OverflowError, User.DoesNotExist):
            user = None
        return user

    def authenticate(self, request, uidb64, token):
        user = self.get_user(uidb64)
        if user is not None and magic_token.check_token(user, token):
            return user
        else:
            return None



from .status_handlers import page_not_found, permission_denied, server_error, unauthorized, bad_request, maintenance
from .login import login
from .logout import logout
from .signup import signup
from .verfiy_email_user import verify_email
from .resend_verification_email_user import resend_verification_code
from .jwt import new_jwt_token
from .tables import user_list, superuser_list, staff_list



from django.http import HttpResponseBadRequest


def ajax_required(function):
    """
    AJAX request required decorator
    use it in your views:

    @ajax_required
    def my_view(request):
        ....

    """

    def wrap(request, *args, **kwargs):
        if not request.is_ajax():
            return HttpResponseBadRequest()
        return function(request, *args, **kwargs)

    return wrap



# Generated by Django 3.0 on 2019-12-18 10:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('CustomAuth', '0005_auto_20191218_1402'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='date_register',
        ),
        migrations.AddField(
            model_name='user',
            name='date_verify',
            field=models.DateTimeField(blank=True, null=True, verbose_name='verify date'),
        ),
    ]



from django.core.exceptions import PermissionDenied


def verify_required(function):
    def wrap(request, *args, **kwargs):
        if request.user.is_anonymous or not request.user.is_verify:
            return
        else:
            return function(request, *args, **kwargs)



from django.http import HttpRequest, HttpResponseRedirect
from django.contrib.auth import logout as user_logout
from django.conf import settings


def logout(request: HttpRequest):
    user_logout(request)
    return HttpResponseRedirect(getattr(settings, 'LOGOUT_REDIRECT', '/'))



from .verify_email_token_generator import VerifyEmailTokenGenerator
from .magic_token_generator import MagicTokenGenerator

account_verify_email_token = VerifyEmailTokenGenerator()
magic_token = MagicTokenGenerator()



# Generated by Django 3.0 on 2019-12-11 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('CustomAuth', '0002_date_joined_auto_now_add'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='wallet',
            field=models.PositiveIntegerField(default=0, verbose_name='Credit of user'),
        ),
    ]



from django.http import HttpResponse, HttpRequest
from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from CustomAuth.models import User


@login_required(login_url='/login/')
def resend_verification_code(request: HttpRequest):
    user: User = request.user
    context = {}
    if not user.is_verify:
        user.verification_email(request)
    else:
        context = {
            'user_already_verified': True
        }
    return render(request, 'CustomAuth/pages/resend_verification_code.html', context=context)



from django.contrib.auth.decorators import user_passes_test
from django.conf import settings


def anonymous_required(function=None, redirect_url=None):
    """
    The way to use this decorator is:
        @anonymous_required
        def my_view(request, pk)
        ...
    """

    if not redirect_url:
        redirect_url = settings.LOGIN_REDIRECT_URL

    actual_decorator = user_passes_test(
        lambda u: u.is_anonymous(),
        login_url=redirect_url
    )

    if function:
        return actual_decorator(function)
    return actual_decorator



import setuptools

with open("CustomAuth/README.md", "r") as fh:
    README = fh.read()

setuptools.setup(
    name="django-custom-user-models",
    version="0.0.82",
    author="mohammad hosein shamsaei",
    author_email="mh.shamsaei.ms@gmail.com",
    description="A custom user model for django authentication",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/MHolger77/django-auth",
    include_package_data=True,
    install_requires=['django-authlib', 'jwt'],
    packages=setuptools.find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.6',
)

# py setup.py sdist bdist_wheel
# twine upload dist/*



from django.contrib.auth.tokens import PasswordResetTokenGenerator


class MagicTokenGenerator(PasswordResetTokenGenerator):
    def _make_hash_value(self, user, timestamp):
        login_timestamp = '' if user.last_login is None else user.last_login.replace(microsecond=100, tzinfo=None)
        return str(user.pk) + str(user.email) + str(user.password) + str(login_timestamp) + str(timestamp)



from django.urls import path
from CustomAuth.views import page_not_found, permission_denied, server_error, unauthorized, bad_request

handler400 = bad_request
handler401 = unauthorized
handler403 = permission_denied
handler404 = page_not_found
handler500 = server_error



from django import forms
from django.contrib.auth.forms import ReadOnlyPasswordHashField, AuthenticationForm, UsernameField

from ..models.user import User


class UserCreationForm(forms.ModelForm):
    """
    A form for creating new users. Includes all the required fields, plus a repeated password.
    """

    password1 = forms.CharField(label='Password', widget=forms.PasswordInput)
    password2 = forms.CharField(label='Password confirmation', widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ('email',)

    def clean_password2(self):
        # Check that the two password entries match
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user: User = super().save(commit=False)
        user.set_password(self.cleaned_data.get("password1"))
        if commit:
            user.save()
        return user


class UserChangeForm(forms.ModelForm):
    """A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    password hash display field.
    """
    password = ReadOnlyPasswordHashField()

    class Meta:
        model = User
        fields = ('email', 'password',
                  'first_name', 'last_name',
                  'is_superuser', 'is_staff', 'is_verify', 'is_active',
                  'user_permissions', 'groups',
                  'date_verify'
                  )

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]


class UserLoginForm(AuthenticationForm):
    user_attr = {
        'autofocus': True,
        'class': ''
    }
    pass_attr = {
        'class': ''
    }
    username = UsernameField(widget=forms.TextInput(attrs=user_attr))
    password = forms.CharField(
        label='Password',
        strip=False,
        widget=forms.PasswordInput(attrs=pass_attr),
    )



from django.contrib.auth import views as auth_views
from django.conf.urls import url

urlpatterns = [
    url(r'^password/reset/$', auth_views.PasswordResetView.as_view(), name='password_reset'),
    url(r'^password/reset/done/$', auth_views.PasswordResetDoneView.as_view(), name='password_reset_done'),
    url(r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        auth_views.PasswordResetConfirmView.as_view(), name='password_reset_confirm'),
    url(r'^reset/done/$', auth_views.PasswordResetCompleteView.as_view(), name='password_reset_complete'),
]



from django.db import models
from django.utils.translation import gettext_lazy as _
from jdatetime import datetime as jalali


class DateMixin(models.Model):
    date_verify = models.DateTimeField(
        _('verify date'),
        blank=True,
        null=True
    )

    date_joined = models.DateTimeField(
        _('join date'),
        blank=True,
        null=True,
        auto_now_add=True
    )

    @property
    def jalali_date_verify(self):
        return jalali.fromgregorian(datetime=self.date_verify).strftime("%d %b %Y")

    def jalali_date_joined(self):
        return jalali.fromgregorian(datetime=self.date_joined).strftime("%d %b %Y")

    class Meta:
        abstract = True

    pass
